@@ -1,5 +1,8 @@
+from __future__ import annotations
+
 from .backend import ALL_BACKENDS, Backend, ExceptionHandling, FunctionHint, FunctionHintSource, register_backend
 from .blob import Blob
+from .cartfile import CARTFile
 from .cgc import CGC, BackedCGC
 from .coff import Coff
 from .elf import ELF, ELFCore, MetaELF
@@ -13,6 +16,7 @@ from .named_region import NamedRegion
 from .pe import PE
 from .region import Region, Section, Segment
 from .regions import Regions
+from .srec import SRec
 from .static_archive import StaticArchive
 from .symbol import Symbol, SymbolSubType, SymbolType
 from .te import TE
@@ -38,6 +42,7 @@ __all__ = [
     "CGC",
     "BackedCGC",
     "Hex",
+    "SRec",
     "Minidump",
     "MachO",
     "NamedRegion",
@@ -55,4 +60,5 @@ __all__ = [
     "SymbolSubType",
     "UefiFirmware",
     "TE",
+    "CARTFile",
 ]
